*----+---1---------2---------3---------4---------5---------6---------7-+
      SUBROUTINE PSKINI( ldebug, ifail )
C  Kinematics for the initial-state (spacelike) parton shower
C  The energy-momentum is precisely conserved.
C  The splitting parameter "z" is interpleted as the ratio of the invariant 
C  masses of the collision after and before the branch.
C
C  In this version, "pt" is determined by the relation
C          pt^2 = (1-z) Q^2
C  derived from the massless and collinear approximation which the 
C  LL subtraction is based on.
C  PLPTN(2,i) is now a parameter of the evolution and not equal to the 
C  virtuality of the evolving parton.
C  2006.07.04, S. Odaka
C
C  The return code "ifail" was added.
C  It is =1 if the kinematics determination fails; i.e. E3 < 0.
C  2007.05.09, S. Odaka
C
      IMPLICIT REAL*8(A-H,O-Z)
      include 'grchad.inc'
      include 'inclm.h'
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP

      dimension ipbeam(2),ippart(2),xpart(2)
      dimension bdir(3),ptemp(4)

      if( ldebug.ge.1 ) print *,'----- PSKINI -----'

C  The return code was added on May 8, 2007.
      ifail= 0

C  Find beam particles
      nbeam= 0
      do 10 i=1,NTOP
         if( NLPTN(2,i).eq.0 ) then
            nbeam= nbeam +1
            if( nbeam.le.2 ) then
               ipbeam(nbeam)= i
            endif
         endif
 10   continue
      if( nbeam.ne.2 ) then
         print *,'PSKINI: abnormal number of beam particles.', nbeam
         stop
      endif

C  The system is assumed to be in the CM frame of beam particles, 
C  where the beam momenta are along the z axis.
      ip1= ipbeam(1)
      ip2= ipbeam(2)
      if( PLPTN(4,ip1).ne.0. .or. PLPTN(5,ip1).ne.0. 
     >     .or. PLPTN(4,ip2).ne.0. .or. PLPTN(5,ip2).ne.0. 
     >     .or. PLPTN(6,ip1)+PLPTN(6,ip2).ne.0. ) then
         print *,'PSKINI: the system is not in the CM frame.'
         stop
      endif
      s0= ( PLPTN(7,ip1) +PLPTN(7,ip2) )**2
      if( ldebug.ge.1 ) then
         print '(5X,''beams at'',I3,'' and'',I3,'': s ='',E12.4)',
     >        ip1,ip2,s0
      endif
      if( s0.lt.0. ) then
         print *,'PSKINI: negative s of the beam collision.',s0
         stop
      endif

C  Starting partons
      do 20 i=1,2
         if( NLPTN(3,ipbeam(i)).ne.1 ) then
            print *,'PSKINI: starting parton is not unique.'
            stop
         endif
         ip= ipbeam(i) +NLPTN(4,ipbeam(i))
         ippart(i)= ip
         xpart(i)= PLPTN(1,ip)
 20   continue

C  Basic parameters of the starting system.
      tau1= xpart(1)*xpart(2)
      if( PLPTN(6,ipbeam(1)).gt.PLPTN(6,ipbeam(2)) ) then
         y1= .5d0*log( xpart(1)/xpart(2) )
      else
         y1= .5d0*log( xpart(2)/xpart(1) )
      endif
      s1= s0*tau1
      roots1= sqrt( s1 )
      if( ldebug.ge.1 ) then
         print '('' Starting x = '',2E12.4)', xpart
         print '(5X,''tau, y, s, root(s)  = '',4E14.6)', 
     >        tau1,y1,s1,roots1
      endif

C  Kinematics are tentatively determined in the CM frame of the 
C  starting partons.
      ip1= ippart(1)
      ip2= ippart(2)
C  The starting partons are assumed to be on-shell.
C      qsq1= ptnmas( NLPTN(1,ip1) )**2
C      qsq2= ptnmas( NLPTN(1,ip2) )**2
C      PLPTN(2,ip1)= qsq1
C      PLPTN(2,ip2)= qsq2
C  The initial virtuality of QCDPS is used as the starting point.
C      qsq1= PLPTN(2,ip1)
C      qsq2= PLPTN(2,ip2)
C      pzsq= ( ( s1 -qsq1 -qsq2 )**2/4.d0 -qsq1*qsq2 )/s1
C      pz= sqrt( pzsq )
C  The starting partons are assumed to be on-shell and massless.
      qsq1= 0.d0
      qsq2= 0.d0
      pz= roots1/2.d0

      PLPTN(4,ip1)= 0.d0
      PLPTN(5,ip1)= 0.d0
      if( PLPTN(6,ipbeam(1)).gt.0.d0 ) then
         PLPTN(6,ip1)= pz
      else
         PLPTN(6,ip1)= -pz
      endif
      PLPTN(4,ip2)= 0.d0
      PLPTN(5,ip2)= 0.d0
      if( PLPTN(6,ipbeam(2)).gt.0.d0 ) then
         PLPTN(6,ip2)= pz
      else
         PLPTN(6,ip2)= -pz
      endif
C      PLPTN(7,ip1)= sqrt( qsq1 +pzsq )
C      PLPTN(7,ip2)= sqrt( qsq2 +pzsq )
C  When massless, ...
      PLPTN(7,ip1)= pz
      PLPTN(7,ip2)= pz
      if( ldebug.ge.1 ) then
         print '(5X,''initial partons'')'
         print '(5X,I3,'': '',4E14.6)', 
     >        ip1,(PLPTN(k,ip1),k=4,7)
         print '(5X,I3,'': '',4E14.6)', 
     >        ip2,(PLPTN(k,ip2),k=4,7)
      endif

C  Loop over branches.
 1    continue

C  Determine which side to branch.
      iside= 0
      q2branch= -1.d15
      do 30 i=1,2
         ipmoth= ippart(i)
C  PS must be spacelike.
         if( NLPTN(6,ipmoth).ne.-1 ) goto 30
         nchild= NLPTN(3,ipmoth)
         if( nchild.gt.0 ) then
            ipch1= ipmoth +NLPTN(4,ipmoth)
C  The virtuality is negative for the evoluting parton, while it is zero
C  for the radiation. 
            q2side= 0.
            do 31 j=1,nchild
               ipch= ipch1 +j -1
               q2= PLPTN(2,ipch)
               if( q2.lt.q2side ) q2side= q2
 31         continue
C  Choose the smaller virtuality side.
            if( q2side.gt.q2branch ) then
               iside= i
               q2branch= q2side
            endif
         endif
 30   continue

C ===========================================================
C  iside = 0 means that we are already at the end of PS.
      if( iside.lt.1 ) then
C  Status code for the final partons.
         NLPTN(5,ippart(1))= 3
         NLPTN(5,ippart(2))= 3
C  Fix the mass of radiations.
         esum= 0.d0
         do i=1,NTOP
            ist= NLPTN(5,i)
            if( ist.eq.4 .or. ist.eq.5 ) then
               PLPTN(7,i)= sqrt( ptnmas( NLPTN(1,i) )**2 
     >              +PLPTN(4,i)**2 +PLPTN(5,i)**2 +PLPTN(6,i)**2 )
               esum= esum +PLPTN(7,i)
            endif
         enddo
C  Remaining partons.
         do ibeam=1,2
            esum= esum +PLPTN(7,ippart(ibeam))
         enddo
C  Adjust the momenta of the starting partons.
C  The system must be in the CM frame of these partons.
C  Though there may be slight inconsistency due to rounding errors, 
C  such inconsistency should be corrected for after adding the final state.
         s1mod= esum**2
         ip1= ipbeam(1) +NLPTN(4,ipbeam(1))
         ip2= ipbeam(2) +NLPTN(4,ipbeam(2))
         qsq1= ptnmas( NLPTN(1,ip1) )**2
         qsq2= ptnmas( NLPTN(1,ip2) )**2
         pzsq= ( ( s1mod -qsq1 -qsq2 )**2/4.d0 -qsq1*qsq2 )/s1mod
         pz= sqrt( pzsq )
         PLPTN(4,ip1)= 0.d0
         PLPTN(5,ip1)= 0.d0
         if( PLPTN(6,ipbeam(1)).gt.0.d0 ) then
            PLPTN(6,ip1)= pz
         else
            PLPTN(6,ip1)= -pz
         endif
         PLPTN(4,ip2)= 0.d0
         PLPTN(5,ip2)= 0.d0
         if( PLPTN(6,ipbeam(2)).gt.0.d0 ) then
            PLPTN(6,ip2)= pz
         else
            PLPTN(6,ip2)= -pz
         endif
         PLPTN(7,ip1)= sqrt( qsq1 +pzsq )
         PLPTN(7,ip2)= sqrt( qsq2 +pzsq )
C  Debug.
         if( ldebug.ge.1 ) then
            print *,'PSKINI: branch completed.'
            print '(5X,''final partons'')'
            print '(5X,I3,'': '',4E14.6)', 
     >           ippart(1),(PLPTN(k,ippart(1)),k=4,7)
            print '(5X,I3,'': '',4E14.6)', 
     >           ippart(2),(PLPTN(k,ippart(2)),k=4,7)
            print '(5X,''final x = '',2E12.4)', xpart
            s= xpart(1)*xpart(2)*s0
            print '(10X,''-> s = '',E14.6)', s
            pxsum= PLPTN(4,ippart(1)) +PLPTN(4,ippart(2))
            pysum= PLPTN(5,ippart(1)) +PLPTN(5,ippart(2))
            pzsum= PLPTN(6,ippart(1)) +PLPTN(6,ippart(2))
            esum= PLPTN(7,ippart(1)) +PLPTN(7,ippart(2))
            ptsum= sqrt( pxsum**2 +pysum**2 )
            s= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            print '(5X,''p, E sum = '',4E14.6)', pxsum,pysum,pzsum,esum
            print '(5X,''s, pt = '',2E14.6)', s,ptsum
         endif
C  Boost the partons to the lab. frame before returning.
C  Boost parameters are determined from the rapidity of the starting 
C  parton system.
         gamma= cosh( y1 )
         beta= tanh( y1 )
         if( ldebug.ge.1 ) then
            print *,'Boost all partons to the lab. frame.'
            print '(5X,''y of the starting parton system = '',
     >           E14.6)', y1
            print '(5X,''beta, gamma = '',2E14.6)', beta,gamma
         endif
         do 91 i=1,ntop
            if( i.eq.ipbeam(1) .or. i.eq.ipbeam(2) ) goto 91
            if( ldebug.ge.3 ) print '(5X,I3,4E14.6)', 
     >           i,(PLPTN(k,i),k=4,7)
            pz= PLPTN(6,i)
            E= PLPTN(7,i)
            PLPTN(6,i)= gamma*( pz +beta*E )
            PLPTN(7,i)= gamma*( beta*pz +E )
            if( ldebug.ge.3 ) print '(5X,'' -> '',4E14.6)', 
     >           (PLPTN(k,i),k=4,7)
 91      continue
         if( ldebug.ge.1 ) then
            print '(5X,''final partons'')'
            print '(10X,I3,'': '',4E14.6)', 
     >           ippart(1),(PLPTN(k,ippart(1)),k=4,7)
            print '(10X,I3,'': '',4E14.6)', 
     >           ippart(2),(PLPTN(k,ippart(2)),k=4,7)
            pxsum= PLPTN(4,ippart(1)) +PLPTN(4,ippart(2))
            pysum= PLPTN(5,ippart(1)) +PLPTN(5,ippart(2))
            pzsum= PLPTN(6,ippart(1)) +PLPTN(6,ippart(2))
            esum= PLPTN(7,ippart(1)) +PLPTN(7,ippart(2))
            s= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            ysum= .5d0*log( ( esum +pzsum )/( esum -pzsum ) )
            ptsum= sqrt( pxsum**2 +pysum**2 )
            print '(5X,''s, y, pt = '',3E14.6)', s,ysum,ptsum
            pxsum= 0.d0
            pysum= 0.d0
            pzsum= 0.d0
            esum= 0.d0
            do 93 i=1,ntop
               if( NLPTN(3,i).lt.1 ) then
                  pxsum= pxsum +PLPTN(4,i)
                  pysum= pysum +PLPTN(5,i)
                  pzsum= pzsum +PLPTN(6,i)
                  esum= esum +PLPTN(7,i)
               endif
 93         continue
            print *,'PSKINI: completed.'
            print '(5X,''total px, py, pz, E = '',4E14.6)', 
     >           pxsum,pysum,pzsum,esum
            ssum= esum**2 -pxsum**2 -pysum**2 -pzsum**2
            ysum= .5d0*log( ( esum +pzsum )/( esum -pzsum ) )
            print '(5X,''s, y = '',3E14.6)', ssum,ysum
         endif

C  Minor branches (timelike evolution of the radiation).
         if( ishower.gt.2 ) then
            do i=1,ntop
               if( NLPTN(5,i).eq.4 ) then
                  iprad= i
                  call psiminor( iprad )
               endif
            enddo
         endif

         return
      endif
C ===========================================================

C  Set the pointers.
      ipmoth= ippart(iside)
      nchild= NLPTN(3,ipmoth)
      if( nchild.ne.2 ) then
         print *,'PSKINI: num. of children is not 2. ',nchild
         stop
      endif
      ipch1= ipmoth +NLPTN(4,ipmoth)
      ipch2= ipch1 +1
      q21= PLPTN(2,ipch1)
      q22= PLPTN(2,ipch2)
      if( q21.lt.q22 ) then
         ipchild= ipch1
         iprad= ipch2
      else
         ipchild= ipch2
         iprad= ipch1
      endif

      if( iside.eq.1 ) then
         iptarget= ippart(2)
      else
         iptarget= ippart(1)
      endif

      if( ldebug.ge.2 ) then
         print '(I3,'' (mother) -> '',I3,'' (child) '',I3,
     >        '' (rad), with '',I3,'' (target): q-sq = '',E12.4)',
     >        ipmoth,ipchild,iprad,iptarget, q2branch
      endif

C  Move to the CM frame of the "mother" and "target".
      call bstcm( PLPTN(4,ipmoth), PLPTN(4,iptarget),
     >     P1, E1, E2, bdir, gamma, phi, theta )

      if( ldebug.ge.2 ) then
         print *,'    PLPTN(1-7,i)'
         print '(10X,''mother: '',7E14.6)', (PLPTN(k,ipmoth),k=1,7)
         print '(10X,''target: '',7E14.6)', (PLPTN(k,iptarget),k=1,7)
         print '(5X,''p1, E1, E2 = '',3E14.6)', P1,E1,E2
         print '(5X,''gamma, phi, theta = '',3E14.6)', gamma,phi,theta
      endif

C  Parameters for the branch.
      zbr= PLPTN(1,ipchild)
      q2abs= -q2branch
      rootshat= E1 +E2
      shat= rootshat**2
      shat1= shat*zbr
      qsq1= E1**2 -P1**2
      qsq2= E2**2 -P1**2
C  The radiation is assumed to be on-shell.
C      qsq4= ptnmas( NLPTN(1,iprad) )**2
C  The radiation is assumed to be massless.
      qsq4= 0.d0

C---------------------------------------------------
C  Determine "PT"
      PT2= ( 1.d0 -zbr )*q2abs
C  (Other options)
CCC      PT2= ( 1.d0 -zbr )**2 * q2abs
CCC      PT2= ( 1.d0 -zbr -q2abs/shat )*q2abs
C---------------------------------------------------
      if( PT2.lt.0.d0 ) then
         if( ldebug.ge.1 ) print *,'!!! pt-sq < 0 ',
     >        '-> set to zero !!!'
         PT2= 0.d0
      endif

      if( ldebug.ge.2 ) then
         print '(5X,''z, pt-sq, qsq1, qsq2, qsq4 = '',5E14.6)',
     >        zbr, PT2, qsq1,qsq2,qsq4
      endif

C  Solution!

C  E3 becomes negative when the Q-value is not large enough to make
C  the radiation.
C  This may happen when the radiation is massive (e.g., b quark).
C  The event weight should be set to zero in this case.
C  May 8, 2007

C      E3= ( shat1 +qsq1 -qsq2 -qsq4 )/2.d0/rootshat
C  When the radiation is massless, ...
      E3= ( shat1 +qsq1 -qsq2 )/2.d0/rootshat
      if( ldebug.ge.2 ) print '(5X,''E3 '',E14.6)',E3
      if( E3.lt.0.d0 ) then
         if( ldebug.ge.1 ) print *,'!!! E3 < 0 -> fail !!!'
         ifail= 1
         return
      endif

C      PT2lim= ( E2 +E3 )**2 -shat1
C  When the radiation is massless, ...
      PT2lim= ( ( 1.d0 -zbr )/2.d0 )**2 * shat
      if( ldebug.ge.2 ) print '(5X,''pt-sq(lim) '',E14.6)',PT2lim
      if( PT2lim.lt.0.d0 ) then
C  It is impossible to derive a reasonable solution if PT2lim is negative.
C  PT2lim is positive definite when the radiation is massless.
         if( ldebug.ge.1 ) print *,'!!! pt-sq(lim) < 0 -> fail !!!'
         ifail= 2
         return
      endif
      if( PT2.gt.PT2lim ) then
         if( ldebug.ge.1 ) print *,'!!! pt-sq > pt-sq(lim) ',
     >        '-> forced to be pt-sq = pt-sq(lim) !!!'
         PT2= PT2lim
      endif
      PL= P1 -sqrt( PT2lim -PT2 )
      PT= sqrt( PT2 )

      if( ldebug.ge.2 ) print '(5X,''=> E3, PL, PT = '',3E14.6)', 
     >     E3,PL,PT

C  Randomize the phi direction.
      rmphi= ( QPRAND(idum)*2.d0 -1.d0 )*pi
      px= PT*cos( rmphi )
      py= PT*sin( rmphi )
      if( ldebug.ge.2 ) then
         print '(5X,''phi, px, py = '',3E14.6)', rmphi,px,py
         snew= ( E3 +E2 )**2 -px**2 -py**2 -( P1 -PL )**2
         print '(10X,''new s = '',E14.6)', snew
      endif

C  Boost back the "child".
      if( ldebug.ge.2 ) print *,'Boost-back the "child".'
      ptemp(1)= px
      ptemp(2)= py
      ptemp(3)= PL
      ptemp(4)= E3
      call bstbk( ptemp, bdir, gamma, phi, theta, PLPTN(4,ipchild) )
      PLPTN(3,ipchild)= PT2

      if( ldebug.ge.2 ) then
         qsqdif= PLPTN(7,ipchild)**2 -PLPTN(4,ipchild)**2
     >        -PLPTN(5,ipchild)**2 -PLPTN(6,ipchild)**2 
     >        -PLPTN(2,ipchild)
         print '(5X,''q-sq. diff = '',E14.6)', qsqdif
      endif

C  Boost back the radiation.
      if( ldebug.ge.2 ) print *,'Boost-back the radiation.'
      ptemp(1)= -px
      ptemp(2)= -py
      ptemp(3)= P1 -PL
      ptemp(4)= E1 -E3
      call bstbk( ptemp, bdir, gamma, phi, theta, PLPTN(4,iprad) )
      PLPTN(3,iprad)= PT2

C  Set the status code.
      NLPTN(5,ipmoth)= 3
      NLPTN(5,iprad)= 4

      if( ldebug.ge.2 ) then
         qsqdif= PLPTN(7,iprad)**2 -PLPTN(4,iprad)**2
     >        -PLPTN(5,iprad)**2 -PLPTN(6,iprad)**2 
     >        -PLPTN(2,iprad)
         print '(5X,''q-sq. diff = '',E14.6)', qsqdif
      endif

      if( ldebug.ge.2 ) then
         print '(5X,''mother: '',4E14.6)', (PLPTN(k,ipmoth),k=4,7)
         print '(5X,''child:  '',4E14.6)', (PLPTN(k,ipchild),k=4,7)
         print '(5X,''rad:    '',4E14.6)', (PLPTN(k,iprad),k=4,7)
         print '(5X,''target: '',4E14.6)', (PLPTN(k,iptarget),k=4,7)
         snew= ( PLPTN(7,ipchild) +PLPTN(7,iptarget) )**2
     >        -( PLPTN(4,ipchild) +PLPTN(4,iptarget) )**2
     >        -( PLPTN(5,ipchild) +PLPTN(5,iptarget) )**2
     >        -( PLPTN(6,ipchild) +PLPTN(6,iptarget) )**2
         print '(10X,''new s = '',E14.6)', snew
      endif

C  Go to the next branch
      xpart(iside)= xpart(iside)*zbr
      ippart(iside)= ipchild
      goto 1

      END
*----+---1---------2---------3---------4---------5---------6---------7-+
      double precision function ptnmas( id )
C
C  Return the parton mass.
C  The values referred to are set in QPINIT.
C
      implicit real*8(a-h,o-z)
      common/QPmass/glmass,qkmass(6)

      ida= iabs( id )
      if( ida.ge.1 .and. ida.le.6 ) then
         ptnmas= qkmass(ida)
      elseif( ida.eq.21 ) then
         ptnmas= glmass
      elseif( ida.eq.22 ) then
         ptnmas= 0.d0
      else
         print *,'PTNMAS: abnormal parton ID. ',id
         stop
      endif

      return
      end

*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine psiminor( iprad )
C  Add branches to the radiations in the initial state.
C  We do not care about the energy conservation, while strictly conserve 
C  the momentum.
C  Jul 28 2011, S. Odaka
C
      implicit real*8(a-h,o-z)
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP

      real*8 bdir(3),psave(4)

C  Copy the radiation information to the top of the PS array.
C  The parton is assumed to have a momentum equal to the pT of the radiation.
      NTOP= NTOP +1
      ipcopy= NTOP
      do i=1,10
         PLPTN(i,ipcopy)= PLPTN(i,iprad)
         NLPTN(i,ipcopy)= NLPTN(i,iprad)
      enddo
      ptrad= sqrt( PLPTN(3,ipcopy) )
      PLPTN(4,ipcopy)= 0.d0
      PLPTN(5,ipcopy)= 0.d0
      PLPTN(6,ipcopy)= ptrad
      PLPTN(7,ipcopy)= ptrad

C  PS branches; return if no branch.
      call QPminor( ipcopy )
      if( NTOP.le.ipcopy ) then
         do i=1,10
            PLPTN(i,ipcopy)= 0.d0
            NLPTN(i,ipcopy)= 0
         enddo
         NTOP= ipcopy -1
         NLPTN(5,iprad)= 5
         return
      endif

C  Jet mass.
      istart= ipcopy +1
      pxtot= 0.d0
      pytot= 0.d0
      pztot= 0.d0
      etot= 0.d0
      do ip=istart,NTOP
         if( NLPTN(5,ip).eq.5 ) then
            pxtot= pxtot +PLPTN(4,ip)
            pytot= pytot +PLPTN(5,ip)
            pztot= pztot +PLPTN(6,ip)
            etot= etot +PLPTN(7,ip)
         endif
      enddo
      amsq= etot**2 -pxtot**2 -pytot**2 -pztot**2

C  Boost to the cm frame, tentatively.
      do i=1,3
         psave(i)= PLPTN(i+3,ipcopy)
      enddo
      psave(4)= sqrt( amsq +psave(1)**2 +psave(2)**2 +psave(3)**2 )
      call bstcm1( psave, bdir, gamma, phi, theta )
      nadd= 0
      do ip=istart,NTOP
         nadd= nadd +1
         iptemp= NTOP +nadd
         call bstfw( PLPTN(4,ip), 
     >        bdir, gamma, phi, theta, PLPTN(4,iptemp) )
      enddo

C  Boost to the initial frame by conserving the jet mass.
      do i=1,3
         psave(i)= PLPTN(i+3,iprad)
      enddo
      psave(4)= sqrt( amsq +psave(1)**2 +psave(2)**2 +psave(3)**2 )
      call bstcm1( psave, bdir, gamma, phi, theta )
      nadd= 0
      do ip=istart,NTOP
         nadd= nadd +1
         iptemp= NTOP +nadd
         call bstbk( PLPTN(4,iptemp), 
     >        bdir, gamma, phi, theta, PLPTN(4,ip) )
      enddo

C  Clean up.
      do ip=NTOP+1,NTOP+nadd
         do i=4,7
            PLPTN(i,ip)= 0.d0
         enddo
      enddo

C  Fix the array for the radiation.
      NLPTN(3,iprad)= NLPTN(3,ipcopy)  ! num. of children
      NLPTN(5,iprad)= 6                ! status code

C  Shift the arrays for branch products to delete the starting info.
      do ip=istart,NTOP
         jp= ip -1
         do i=1,10
            PLPTN(i,jp)= PLPTN(i,ip)
            NLPTN(i,jp)= NLPTN(i,ip)
         enddo
      enddo
      do i=1,10
         PLPTN(i,NTOP)= 0.d0
         NLPTN(i,NTOP)= 0
      enddo
      NTOP= NTOP -1
      istart= istart -1

C  Fix the relative pointers.
      NLPTN(4,iprad)= istart -iprad
      NLPTN(2,istart)= istart -iprad

      return
      end
*----+---1---------2---------3---------4---------5---------6---------7-+
      subroutine QPminor( ip )
C  Apply PS to a radiation.
C  Details:
C    Apply the PS to those partons having the status code of 4.
C    The energy scale of the PS is assumed to be the "pt" of the radiation.
C    If no branch, the status code is changed to 5 (end point), and 
C    the element added by QCDPSf is removed.
C    If further branches, the status code is changed to 6.
C  Oct 21 2009, S. Odaka
C
C  Modification according to the new scheme of QCDPSf.
C    Now we do not need a recursive call of QCDPSf.
C    The "minor branch" is no longer applied to the final state.
C    Thus, this routine is called only for the initial-state radiations in 
C    "pskini".
C  12 May 2010, S.O.
C
C  Moved from QCDPSf.f to pskini.f.
C  28 Jun 2010, S.O.
C
C  The energy fixing was removed because we do not care about the energy 
C  conservation, now.
C  28 Jul 2011, S.O.
C
      implicit real*8(a-h,o-z)
      COMMON/QPLIST/PLPTN(10,10000),NLPTN(10,10000),NTOP

      npre= NTOP
C      Escale= sqrt( PLPTN(3,ip) )
C  1 Aug 2011, S.O.
      Escale= sqrt( PLPTN(2,ip) )
      Call QCDPSf( ip, Escale, wgt)

      if( NTOP.le.npre+1 ) then
         NLPTN(3,ip)= 0
         NLPTN(4,ip)= 0
         NLPTN(5,ip)= 5
         do i=1,10
            PLPTN(i,NTOP)= 0.d0
            NLPTN(i,NTOP)= 0
         enddo
         NTOP= NTOP -1
      else
         NLPTN(5,ip)= 6
         ipnew= ip +NLPTN(4,ip)
         call pskfin( ipnew )
      endif

      return
      end
